<?php
/*
 * Filename: backup_monitor.php
 *                  
 * Description: a PHP script that monitor's various backup files in different systems, provides email alerts
 *               when file times are out of range or need warning
 
 * Created on December 23, 2013 by Tony Brandao abrandao29@gmail.com
 *
 * @Current Version    $Revision: $:
 * @Last Changed By    $Author: abrandao $:
 * @Last Changed On    $Date: 2013/12/05 15:21:08 $:
 * File Version        $$
 */
 /* Use different settings when setup in cron mode ; such as through a cron job */
 date_default_timezone_set('America/New_York');  /* required for file dates */
 
 /* When using command line other defined function must come first */
if (isset($_SERVER['argc'])) 
    define('CLI', true);
	 else 
    define('CLI', false);
 
 //include "backup_ssh.php"; /*class for ssh connections which require encryption support */
 
 //USER Configurable Array for file information
  $age_warn_limit=  3600* 24 ; //default to one day
  $age_alert_limit= 3600* 24   ; //age limit before alerts sent out defaults to two days
  $email=array();
  $email["alerts_to"]="recipient@domain.com"; //default address to send emails to, can be changed on a per 	file basis - see array 
 
  /* Array configuration file and format to monitor files */
  /* Csv File format: file to be setup  */
 $monitor_config_file=null;  /* configuration file name,  null to use array below */

 //Array Format as follows:
 // section | type|protocol://host_name | url/path | description | username | password | warn_limit | alert_limit | email_to | size_alert_limit_bytes
  $files_to_monitor = array( 
    array("Local Files","file://" , "/var/www/html/lab/back/backups.php","PHP File sample " ,null,null,  $age_warn_limit , $age_alert_limit*2, $email["alerts_to"],1 ),
	array("Local Files","file://" , "/var/www/html/lab/back/backup_monitor.php","PHP File sample " ,null,null,  $age_warn_limit , $age_alert_limit*2, $email["alerts_to"],1 ),
	 array("Local Files","file://" , "/mnt/var/www/db/db.sql","Recent Database backup " ,null,null,  $age_warn_limit , $age_alert_limit*2, $email["alerts_to"],1 ),
	/* Nas Drive */ 
	array("Local Nas","file://" , "/mnt/var/www/db/users.log","Recent Database log " ,null,null,  $age_warn_limit , $age_alert_limit*2, $email["alerts_to"],1 ),
   array("Local Nas","file://" , "/var/www/html/lab/back/backups.php","PHP File sample " ,null,null,  $age_warn_limit , $age_alert_limit*2, $email["alerts_to"],1 ),
 /* Remote Nas */
	array("Remote Nas","file://" , "/mnt/remote/var/www/db/users.log","Recent Database log " ,null,null,  $age_warn_limit , $age_alert_limit*2, $email["alerts_to"],1 )

); //end of array


/*  email Alerts are sent via */ 
$email["alerts_enabled"]=CLI;  /* true=turns on email alerts, tied to cron  command line on */
$email["method"] = "mail";  /* Options: mail | smtp */

/* use with PHPMailer or similar  */
$email["smtp_user"] = "server_user_name";  
$email["smtp_pass"] ="server_user_pass";
$email["smtp_server"]="server_ip_hostname"; 
$email["smtp_from_user"]="alerts@hostname.com";
$email["smtp_from_name"]="Backup Monitor Alerts";


$email["subject"] ="Backup Alert --[ {filename} ]--";
$email["message_placeholders"] ="
The backup monitor running at: {time_now}\n
Description: {file_description} \n
Filename: {filename} \n
File Details: ({file_size} bytes) last modified at {filedatetime} .
Backup Age: {file_elapsedtime} > ( limit: {backup_alerttime} seconds )\n
------------------------------------------------\n
";

/* Are we using console/command line (such as cron) invoke the function directly */
if (CLI==true) {
	echo "Running backup Monitor - Command Line \n ----------------------------\n";
	main_backup_monitor("table_mon");  //call the function directly
	echo "\n End Running backup Monitor - Command Line \n ";
} else {
	if (isset($_REQUEST['d']) && $_REQUEST['d']=="t")
	{
	echo "Running backup Monitor - via direct Web request\n";
	main_backup_monitor("table_mon");  //running directly via a web request
	}
}


/**********************************************************************
   email_alert sends an email with notification
	
    @$email_to  whom to send the email to
	@$message  what the email message is 
    @return null
*/

function email_alert($email_to=null, $message=null, $row=null)
{
global $email; //global array holds basic info
static $last_email_to=null;  //track last email recipient consolidate multiple emails to same recipient
$result=false;
$msg_template=$email["message_placeholders"];

if ( $email["alerts_enabled"]==false) return $result;  //exit when email alerts are disabled

//substitute the email message with all the penitent place holders from the row
$msg_template= str_replace("{time_now}",date("m/d/y h:i:s"), $msg_template);
$msg_template= str_replace("{filename}",$row["filename"], $msg_template);
$msg_template= str_replace("{filedatetime}",$row["file_date_and_time"], $msg_template);
$msg_template= str_replace("{file_elapsedtime}",$row["file_age"], $msg_template);
$msg_template= str_replace("{backup_alerttime}",$row["alert_limit"], $msg_template);
$msg_template= str_replace("{file_size}",$row["file_size"], $msg_template);
$msg_template= str_replace("{file_description}",$row["description"], $msg_template);

$message=$msg_template." ".$message;

$subject= str_replace("{filename}",basename($row["filename"]), $email["subject"] );

if (isset($email_to) && isset($subject) )
if ( $email["method"]=="smtp")
 {
  
  include_once("phpmailer/class.phpmailer.php");
  $mailer = new PHPMailer();
  $mailer->From = $email["smtp_from_user"];
  $mailer->FromName = $email["smtp_from_name"];
  $mailer->Host = $email["smtp_server"];  //local exchange server
  $mailer->SMTPSecure = "tls";  		// these settings particular to EXCHANGE
  $mailer->SMTPAuth  = true;
  $mailer->IsSMTP(); // telling the class to use SMTP
  $mailer->IsHTML(false); // HTML or Text message
  $mailer->Username  = $email["smtp_user"];
  $mailer->Password  = $email["smtp_pass"];;
  $mailer->ErrorInfo=null;
  $reply_to_email=$mailer->From ;
  $reply_to_name=$mailer->FromName;
  
  // Set the subject
    $mailer->AddAddress($email_to, $email_to);
	$mailer->Subject = $subject;
	$mailer->Body=$message;
	if(!$mailer->Send()) //send the mail
	 {
	 echo "Mail error:".$mailer->ErrorInfo;
	 $result=false;
	 }
	 else
	 $result=true;
  //smtp mail
  }
 else
 {
   $result=mail($email_to,$subject, $message);
  
 }
 
 return $result;
} //end function 


/**********************************************************************
   humanTiming take a time and return the difference from now in human readable 
    @$time  file time 
    @return complete time  as human friendly string like 1 day 2 hours 13 minutes 2 second string
*/

function humanTiming ($time)
{
    // to get the time since that moment
    $time = time() - $time;
	// time unit constants
    $timeUnits = array (
        31536000 => 'year',
        2592000 => 'month',
        604800 => 'week',
        86400 => 'day',
        3600 => 'hour',
        60 => 'minute',
        1 => 'second'
    );

    // iterate over time contants to build a human 
    $humanTiming=null;
	
    foreach ($timeUnits as $unit => $text)
    {
        if ($time < $unit)
            continue;
        $numberOfUnits = floor($time / $unit);

        // human readable token for current time unit
        $humanTiming = $humanTiming.' '.$numberOfUnits.' '.$text.(($numberOfUnits>1)?'s':'');

        // compute remaining time for next loop iteration
        $time -= $unit*$numberOfUnits;
    }
    return $humanTiming;
}

/**********************************************************************
   build_status_table with various files and information
	
    @$row  array containing row cells
	@$table_end  is this the last row, if so close the table
	@$elapsed_time how much time (in second has elapsed) since file was created
    @return complete HTML table as string
*/

function build_status_table($row,$table_end=false,$css_class=null)
{
static $table_start=false;  
static $table=null;
global $table_header_row, $age_warn_limit, $age_alert_limit;  //defined in the configuration section
static $last_section=null;

if ( !$table_start) /* initializes the table if false */
 {
 $table="\n<table border=1 class='$css_class'>\n"; 
 $table.="<caption>Backup Monitor Status Grid</caption>\n";
 $table.= "<THEAD><tr>";
 foreach($row as $i=> $value)  //build the header row names from the array elements
	{
	if ($i!="section")  //skip section heading
	$table.="<th>$i</th>\n";
	}
 $table.="</tr></THEAD>\n<TBODY>\n";
 $table_start=true;  
 }

 if (isset($row) && is_array($row) )
 {
 
   if ($row["section"]!= $last_section)
    {
	$table.="\n<tr class='table_section'><td colspan='10'>".$row["section"]."</td></tr>\n<tr>";  //for new sections, add section as new row
	$last_section=$row["section"];
	}
	else
    $table.="\n<tr>";
	
	
	foreach($row as $i=> $value)
	{
	// <span class="label bg-yellow text-white">$value</span>
	switch ($i) 
	{
	case "section" : break;
				
	case "connection" : 
				if ($value==true )
				$table.="<td><span class='label label-success '>Exists</span></td>\n";
				else
				$table.="<td><span class='label label-danger'>Not Found</span></td>\n";
				break;
				
	case "elapsed_time" : 
					if ($value<= 0)
					$table.="<td><span class='label label-danger'>".round($value/(3600*24),1)."d </span></td>\n";
					elseif ($value<= $row["alert_limit"])
					$table.="<td><span class='label label-success'>".round($value/(3600*24),1)."d </span></td>\n";
					else
					$table.="<td><span class='label label-danger'>".round($value/(3600*24),1)."d </span></td>\n";
					break;					
				

	case "age_alert" : 
					if ($value==true )
					$table.="<td><span class='label label-danger'>Alert</span></td>\n";
					else
					$table.="<td><span class='label label-success'>Ok</span></td>\n";
					break;				

	case "file_size" : 
					if ($value==0)
					$table.="<td><span class='label label-danger'>$value b</span></td>\n";
					else
					$table.="<td><span class='label label-success'>$value b</span></td>\n";
					break;				
					
case "alert_limit" : 
					$table.="<td>".($value/(3600*24))."d </td>\n";
					break;							
case "filename" : 
					$table.="<td>".$value." </td>\n";
					
					break;							

	default :
	$table.="<td>$value</td>\n";
	}
	
	} //end of foreach
	
	$table.="</tr>";  //end of row
 }
 
if ($table_end==true)
 {
 $table.="\n</TBODY></table>";
 $table_start=false; //revert back to begin another table
 }
 
return $table;
}

/**********************************************************************
    Main backup_monitor function
*/

function main_backup_monitor($css_class=null)
{
global $files_to_monitor,$monitor_config_file;

if ( isset($monitor_config_file))  /* is the configuration file-name specified ? */
  {
   if (file_exists($monitor_config_file) )
    {
	//Read the file and build up the 	$files_to_monitor array with file contents
	$files_to_monitor = fopen($monitor_config_file, "r");
	
	}
   else
   die("File not found: $monitor_config_file");
   
  } //end if using a cvs config file
 
/* now begin looping through the array and displaying the results */
foreach( $files_to_monitor as $i => $file_test )
{
if (!is_array($file_test))
    {
        echo $i ." => ". $file_test ."\r\n" ;
    }
else
 {
         // print_r($file_test);
		 $data_row=array();  //create an emtpy array
		 $data_row["section"]=$file_test[0]; //presume no valid connection exists
		 $data_row["connection"]=false; //presume no valid connection exists
		 $filename=$file_test[2];  // get the filename
		 $data_row["filename"]= $filename; //get the filename
		 $data_row["description"]= $file_test[3]; //get the filename
		 $data_row["connection"]=false; //assume no connection
		 
		   switch ($file_test[1])  //test connection based on filetime
			{
			  case "file://":  
							if (@file_exists($file_test[2]) )
							   $data_row["connection"]=true;
							  break;
				
			  case "https://":
			  case "http://": $handle = @fopen($file_test[2], "r"); //specify full path
							if (isset($handle))
							  $data_row["connection"]=true;
							break;
							
			  case "smb":  
							if (@file_exists("\\".$file_test[2]) )
							    $data_row["connection"]=true;
							break;
							
			  case "ssh":  /*
										  //ssh connection
							 if(function_exists('ssh2_connect'))
							{
							$connection = ssh2_connect($host, $port);
							$handle = fopen("ssh2.sftp://$sftp".$file, "r");  //open the file
							//
							$ssh=new NiceSSH();
							$ssh->connect();
							}
							else
							die("SSH Function is not defined/available in this version of PHP");
							*/
							break;
		   
		   };
  
		
		// check file date  calculate file date/time
		 if ($data_row["connection"])   //if the file exists (we have a good connection )
		 {
		 $data_row["file_size"]=filesize($filename);
		 $data_row["file_date_and_time"]=date ("m-d-Y H:i:s", @filemtime($filename) );

		$elapsedTime_secs = time() -  filectime($filename); //elapsed time in seconds
				$data_row["file_age"]=humanTiming( filectime($filename) );
		$data_row["age_alert"]=false;
	    $data_row["elapsed_time"]=$elapsedTime_secs;

		$data_row["alert_limit"]=$file_test[7];
		//check alert date
 		if($elapsedTime_secs >=$file_test[7])  //trigger an alert email
		   $data_row["age_alert"]=true;
		 } //end if exists
		 else
		 {
		 $data_row["file_size"]=0;
		 $data_row["file_date_and_time"]="n/a";
		 $data_row["file_age"]="n/a";
	     $data_row["age_alert"]=true;
		 $data_row["elapsed_time"]="n/a";	
    	 $data_row["alert_limit"]=$file_test[7];
		 //$data_row["warn_limit"]=$file_test[6];
		 }
		
		$data_row["email_to"]=$file_test[8];
		if ($data_row["age_alert"] )
		  {
		   $result=email_alert($file_test[8], "=== End of Alert Eamil == ",$data_row); //alerts are immediate at the moment.
		   
		   if ($result)
		    $data_row["email_to"]=$data_row["email_to"]." <span class='label label-info '>sent</span>";
			else
			$data_row["email_to"]=$data_row["email_to"]." <span class='label label-warning '>failed</span>";
		   //TODO setup Alert email digest so alerts are all sent out once
		  }
		//add row to the table
		 build_status_table($data_row,false,$css_class);
  	   
	  	  
 }  //else end 
  
} //foreach

 echo build_status_table(null,true); //close out the status table
 
  
}  //end of function
 
 
?>
